@@ -20,6 +20,7 @@ from .gui.class_gui import (
     DraggableClassGui,
     ButtonClassGui, 
     CollapsibleClassGui,
+    HCollapsibleClassGui,
     SplitClassGui,
     TabbedClassGui,
     StackedClassGui,
@@ -27,6 +28,7 @@ from .gui.class_gui import (
     ListClassGui,
     )
 from .gui._base import PopUpMode, ErrorMode, defaults, MagicTemplate, check_override
+from .gui.toolbar import ToolBarGui
 from .gui import ContextMenuGui, MenuGui, MenuGuiBase
 from ._app import get_app
 
@@ -59,6 +61,7 @@ class WidgetType(Enum):
     draggable = "draggable"
     split = "split"
     collapsible = "collapsible"
+    hcollapsible = "hcollapsible"
     button = "button"
     toolbox = "toolbox"
     tabbed = "tabbed"
@@ -71,7 +74,7 @@ class WidgetType(Enum):
 WidgetTypeStr = Union[Literal["none"], Literal["scrollable"], Literal["draggable"], Literal["split"],
                       Literal["collapsible"], Literal["button"], Literal["toolbox"], Literal["tabbed"],
                       Literal["stacked"], Literal["list"], Literal["subwindows"], Literal["groupbox"],
-                      Literal["mainwindow"]]
+                      Literal["mainwindow"], Literal["hcollapsible"]]
 
 PopUpModeStr = Union[Literal["popup"], Literal["first"], Literal["last"], Literal["above"],
                      Literal["below"], Literal["dock"], Literal["parentlast"]]
@@ -84,6 +87,7 @@ _TYPE_MAP = {
     WidgetType.draggable: DraggableClassGui,
     WidgetType.split: SplitClassGui,
     WidgetType.collapsible: CollapsibleClassGui,
+    WidgetType.hcollapsible: HCollapsibleClassGui,
     WidgetType.button: ButtonClassGui,
     WidgetType.toolbox: ToolBoxClassGui,
     WidgetType.tabbed: TabbedClassGui,
@@ -125,6 +129,7 @@ def magicclass(class_: type | None = None,
                popup_mode: PopUpModeStr | PopUpMode = None,
                error_mode: ErrorModeStr | ErrorMode = None,
                widget_type: WidgetTypeStr | WidgetType = WidgetType.none,
+               icon_path: str = None,
                parent = None
                ):
     """
@@ -159,6 +164,8 @@ def magicclass(class_: type | None = None,
         Option of how to raise errors during function calls.
     widget_type : WidgetType or str, optional
         Widget type of container.
+    icon_path : str, optional
+        Path to the icon image.
     parent : magicgui.widgets._base.Widget, optional
         Parent widget if exists.
     
@@ -178,7 +185,7 @@ def magicclass(class_: type | None = None,
         
     widget_type = WidgetType(widget_type)
     
-    def wrapper(cls) -> ClassGui:
+    def wrapper(cls) -> type[ClassGui]:
         if not isinstance(cls, type):
             raise TypeError(f"magicclass can only wrap classes, not {type(cls)}")
         elif is_dataclass(cls):
@@ -230,11 +237,12 @@ def magicclass(class_: type | None = None,
             if widget_type in (WidgetType.collapsible, WidgetType.button):
                 self.btn_text = self.name
             
-            # TODO: does this "if" make sense?
-            if self.__magicclass_parent__ is None:
-                macrowidget = self.macro.widget.native
-                macrowidget.setParent(self.native, macrowidget.windowFlags())
-                self.macro.widget.__magicclass_parent__ = self
+            macrowidget = self.macro.widget.native
+            macrowidget.setParent(self.native, macrowidget.windowFlags())
+            self.macro.widget.__magicclass_parent__ = self
+            
+            if icon_path:
+                self.icon_path = icon_path
 
         newclass.__init__ = __init__
         
@@ -242,8 +250,11 @@ def magicclass(class_: type | None = None,
         newclass.__repr__ = oldclass.__repr__
         
         return newclass
-    
-    return wrapper if class_ is None else wrapper(class_)
+
+    if class_ is None:
+        return wrapper
+    else:
+        return wrapper(class_)
 
 
 def magicmenu(class_: type = None, 
@@ -252,6 +263,7 @@ def magicmenu(class_: type = None,
               popup_mode: str | PopUpMode = None,
               error_mode: str | ErrorMode = None,
               labels: bool = True, 
+              icon_path: str = None,
               parent = None
               ):
     """
@@ -265,6 +277,7 @@ def magiccontext(class_: type = None,
                  popup_mode: str | PopUpMode = None,
                  error_mode: str | ErrorMode = None,
                  labels: bool = True, 
+                 icon_path: str = None,
                  parent=None
                  ):
     """
@@ -272,6 +285,20 @@ def magiccontext(class_: type = None,
     """    
     return _call_magicmenu(**locals(), menugui_class=ContextMenuGui)
 
+def magictoolbar(class_: type = None, 
+                 *, 
+                 close_on_run: bool = None,
+                 popup_mode: str | PopUpMode = None,
+                 error_mode: str | ErrorMode = None,
+                 labels: bool = True, 
+                 icon_path: str = None,
+                 parent = None
+                 ):
+    """
+    Decorator that can convert a Python class into a menu bar.
+    """    
+    return _call_magicmenu(**locals(), menugui_class=ToolBarGui)
+
 class MagicClassFactory:
     """
     Factory class that can make any magic-class.
@@ -284,6 +311,7 @@ class MagicClassFactory:
                  popup_mode: str | PopUpMode = None,
                  error_mode: str | ErrorMode = None,
                  widget_type: str | WidgetType = WidgetType.none,
+                 icon_path: str = None,
                  parent = None,
                  attrs: dict[str] = None
                  ):
@@ -294,6 +322,7 @@ class MagicClassFactory:
         self.popup_mode = popup_mode
         self.error_mode = error_mode
         self.widget_type = widget_type
+        self.icon_path = icon_path
         self.parent = parent
         self.attrs = attrs
     
@@ -301,19 +330,28 @@ class MagicClassFactory:
         _cls = type(self.name, (), self.attrs)
         cls = magicclass(_cls, layout=self.layout, labels=self.labels, close_on_run=self.close_on_run, 
                          name=self.name, popup_mode=self.popup_mode, error_mode=self.error_mode,
-                         widget_type=self.widget_type, parent=self.parent)
+                         widget_type=self.widget_type, icon_path=self.icon_path, parent=self.parent)
         return cls
     
+    def as_magictoolbar(self) -> ToolBarGui:
+        _cls = type(self.name, (), self.attrs)
+        cls = magictoolbar(_cls, close_on_run=self.close_on_run, popup_mode=self.popup_mode, 
+                           error_mode=self.error_mode, labels=self.labels, icon_path=self.icon_path,
+                           parent=self.parent)
+        return cls   
+    
     def as_magicmenu(self) -> MenuGui:
         _cls = type(self.name, (), self.attrs)
         cls = magicmenu(_cls, close_on_run=self.close_on_run, popup_mode=self.popup_mode, 
-                        error_mode=self.error_mode, labels=self.labels, parent=self.parent)
+                        error_mode=self.error_mode, labels=self.labels, icon_path=self.icon_path,
+                        parent=self.parent)
         return cls
     
     def as_magiccontext(self) -> ContextMenuGui:
         _cls = type(self.name, (), self.attrs)
         cls = magiccontext(_cls, close_on_run=self.close_on_run, popup_mode=self.popup_mode, 
-                           error_mode=self.error_mode, labels=self.labels, parent=self.parent)
+                           error_mode=self.error_mode, labels=self.labels, icon_path=self.icon_path,
+                           parent=self.parent)
         return cls
 
 
@@ -322,6 +360,7 @@ def _call_magicmenu(class_: type = None,
                     popup_mode: str | PopUpMode = None,
                     error_mode: str | ErrorMode = None,
                     labels: bool = True, 
+                    icon_path: str = None,
                     parent = None,
                     menugui_class: type[MenuGuiBase] = None,
                     ):
@@ -354,7 +393,7 @@ def _call_magicmenu(class_: type = None,
     if popup_mode in (PopUpMode.above, PopUpMode.below, PopUpMode.first, PopUpMode.last):
         raise ValueError(f"Mode {popup_mode.value} is not compatible with Menu.")
     
-    def wrapper(cls) -> menugui_class:
+    def wrapper(cls) -> type[menugui_class]:
         if not isinstance(cls, type):
             raise TypeError(f"magicclass can only wrap classes, not {type(cls)}")
         elif is_dataclass(cls):
@@ -393,6 +432,13 @@ def _call_magicmenu(class_: type = None,
             
             if hasattr(self, _POST_INIT):
                 self.__post_init__()
+            
+            if icon_path:
+                self.icon_path = icon_path
+            
+            macrowidget = self.macro.widget.native
+            macrowidget.setParent(self.native, macrowidget.windowFlags())
+            self.macro.widget.__magicclass_parent__ = self
 
         newclass.__init__ = __init__
         
