@@ -1,25 +1,30 @@
 from __future__ import annotations
 from functools import wraps as functools_wraps
-from typing import Any, Callable, TYPE_CHECKING, Iterable, TypeVar, overload, MutableSequence
+from typing import Any, Callable, TYPE_CHECKING, Iterable, Iterator, TypeVar, overload, MutableSequence
 from typing_extensions import _AnnotatedAlias
 import inspect
+import warnings
+import os
 from enum import Enum
 import warnings
 from docstring_parser import parse, compose
+from qtpy.QtGui import QIcon
 
 from magicgui.events import Signal
 from magicgui.signature import MagicParameter
-from magicgui.widgets import FunctionGui, FileEdit, EmptyWidget, Widget, Container
-from magicgui.widgets._bases import ValueWidget
+from magicgui.widgets import FunctionGui, FileEdit, EmptyWidget, Widget, Container, Image, Table, Label
+from magicgui.application import use_app
+from magicgui.widgets._bases.widget import Widget
+from magicgui.widgets._bases import ButtonWidget, ValueWidget
 from macrokit import Expr, Head, Symbol, symbol
 
 from .keybinding import as_shortcut
-from .mgui_ext import AbstractAction, Action, FunctionGuiPlus, PushButtonPlus
-from .utils import get_parameters
+from .mgui_ext import AbstractAction, Action, FunctionGuiPlus, PushButtonPlus, _LabeledWidgetAction, mguiLike
+from .utils import get_parameters, define_callback
 from ._macro import GuiMacro
 
 from ..utils import get_signature, iter_members, extract_tooltip, screen_center
-from ..widgets import Separator
+from ..widgets import Separator, FreeWidget
 from ..fields import MagicField
 from ..signature import MagicMethodSignature, get_additional_option
 from ..wrappers import upgrade_signature
@@ -58,7 +63,7 @@ _RESERVED = {"__magicclass_parent__", "__magicclass_children__", "_close_on_run"
 
 def check_override(cls: type):
     """
-    Some of the methods should not be overriden because they are essential for Magic class
+    Some of the methods should not be overriden because they are essential for magic class
     construction.
 
     Parameters
@@ -92,6 +97,7 @@ class MagicTemplate:
     enabled: bool
     gui_only: bool
     height: int
+    icon_path: str
     label_changed: Signal
     label: str
     layout: str
@@ -139,13 +145,20 @@ class MagicTemplate:
     
     def append(self, widget: Widget) -> None:
         return self.insert(len(self, widget))
-        
-    def insert(self, key: int, widget: Widget) -> None:
+    
+    def _fast_insert(self, key: int, widget: Widget) -> None:
         raise NotImplementedError()
     
+    def insert(self, key: int, widget: Widget) -> None:
+        self._fast_insert(key, widget)
+        self._unify_label_widths()
+    
     def render(self) -> "np.ndarray":
         raise NotImplementedError()
     
+    def _unify_label_widths(self):
+        raise NotImplementedError()
+    
     @property
     def macro(self) -> GuiMacro:
         if self.__magicclass_parent__ is None:
@@ -487,7 +500,7 @@ class MagicTemplate:
                                       "widget in napari."
                                 warnings.warn(msg, UserWarning)
                             
-                            else:    
+                            else:
                                 from qtpy.QtWidgets import QDockWidget
                                 from qtpy.QtCore import Qt
                                 dock = QDockWidget(_get_widget_name(widget), self.native)
@@ -561,6 +574,136 @@ class BaseGui(MagicTemplate):
         self._popup_mode = popup_mode
         self._error_mode = error_mode
         self._my_symbol = Symbol.var("ui")
+        self._icon_path = None
+
+class ContainerLikeGui(BaseGui, mguiLike, MutableSequence):
+    _component_class = Action
+    changed = Signal(object)
+    _list: list[AbstractAction | ContainerLikeGui]
+        
+    @property
+    def icon_path(self):
+        return self._icon_path
+    
+    @icon_path.setter
+    def icon_path(self, path: str):
+        path = str(path)
+        if os.path.exists(path):
+            icon = QIcon(path)
+            if hasattr(self.native, "setIcon"):
+                self.native.setIcon(icon)
+            else:
+                self.native.setWindowIcon(icon)
+            self._icon_path = path
+        else:
+            warnings.warn(
+                f"Path {path} does not exists. Could not set icon.",
+                UserWarning
+            )
+        
+    def _create_widget_from_field(self, name: str, fld: MagicField):
+        cls = self.__class__
+        if fld.not_ready():
+            try:
+                fld.decode_string_annotation(cls.__annotations__[name])    
+            except (AttributeError, KeyError):
+                pass
+            
+        fld.name = fld.name or name.replace("_", " ")
+        action = fld.get_action(self)
+            
+        if action.support_value:
+            # If the field has callbacks, connect it to the newly generated widget.
+            for callback in fld.callbacks:
+                # funcname = callback.__name__
+                action.changed.connect(define_callback(self, callback))
+                     
+            # By default, set value function will be connected to the widget.
+            if fld.record:
+                f = value_widget_callback(self, action, name, getvalue=type(fld) is MagicField)
+                action.changed.connect(f)
+                
+        return action
+    
+    
+    def __getitem__(self, key: int | str) -> ContainerLikeGui | AbstractAction:
+        if isinstance(key, int):
+            return self._list[key]
+        
+        out = None
+        for obj in self._list:
+            if obj.name == key:
+                out = obj
+                break
+        if out is None:
+            raise KeyError(key)
+        return out
+    
+    def __setitem__(self, key, value):
+        raise NotImplementedError()
+    
+    def __delitem__(self, key: int | str) -> None:
+        self.native.removeAction(self[key].native)
+    
+    def __iter__(self) -> Iterator[ContainerLikeGui | AbstractAction]:
+        return iter(self._list)
+    
+    def __len__(self) -> int:
+        return len(self._list)
+    
+    def append(self, obj: Callable | ContainerLikeGui | AbstractAction) -> None:
+        return self.insert(len(self._list), obj)
+    
+    def _unify_label_widths(self):
+        _hide_labels = (_LabeledWidgetAction, ButtonWidget, FreeWidget, Label, FunctionGui,
+                        BaseGui, Image, Table, Action)
+        need_labels = [w for w in self if not isinstance(w, _hide_labels)]
+        
+        if self.labels and need_labels:
+            measure = use_app().get_obj("get_text_width")
+            widest_label = max(measure(w.label) for w in need_labels)
+            for w in need_labels:
+                labeled_widget = w._labeled_widget()
+                if labeled_widget:
+                    labeled_widget.label_width = widest_label
+
+    def render(self):
+        try:
+            import numpy as np
+        except ImportError:
+            raise ModuleNotFoundError(
+                "could not find module 'numpy'. "
+                "Please `pip install numpy` to render widgets."
+            ) from None
+        import qtpy
+        img = self.native.grab().toImage()
+        bits = img.constBits()
+        h, w, c = img.height(), img.width(), 4
+        if qtpy.API_NAME == "PySide2":
+            arr = np.array(bits).reshape(h, w, c)
+        else:
+            bits.setsize(h * w * c)
+            arr = np.frombuffer(bits, np.uint8).reshape(h, w, c)
+
+        return arr[:, :, [2, 1, 0, 3]]
+
+    def _repr_png_(self):
+        """Return PNG representation of the widget for QtConsole."""
+        from io import BytesIO
+
+        try:
+            from imageio import imsave
+        except ImportError:
+            print(
+                "(For a nicer magicmenu widget representation in "
+                "Jupyter, please `pip install imageio`)"
+            )
+            return None
+
+        with BytesIO() as file_obj:
+            imsave(file_obj, self.render(), format="png")
+            file_obj.seek(0)
+            return file_obj.read()
 
 def _get_widget_name(widget: Widget):
     # To escape reference
@@ -757,14 +900,18 @@ def _child_that_has_widget(self: BaseGui, method: Callable,
             
 
 def _method_as_getter(self, bound_value: Callable):
-    *clsnames, funcname = bound_value.__qualname__.split(".")
+    qualname = bound_value.__qualname__
+    _locals = "<locals>."
+    if _locals in qualname:
+        qualname = qualname.split(_locals)[-1]
+    *clsnames, funcname = qualname.split(".")
     
     def _func(w):
         ins = self
         while clsnames[0] != ins.__class__.__name__:
             ins = getattr(ins, "__magicclass_parent__", None)
             if ins is None:
-                raise ValueError(f"Method {bound_value.__qualname__} is invisible"
+                raise ValueError(f"Method {qualname} is invisible"
                                  f"from magicclass {self.__class__.__qualname__}")
         
         for clsname in clsnames[1:]:
@@ -821,7 +968,6 @@ def value_widget_callback(gui: MagicTemplate, widget: ValueWidget, name: str, ge
         # >>> x.name = value
         target = Expr(Head.getattr, [symbol(gui), sub])
         expr = Expr(Head.assign, [target, widget.value])
-        
         if gui.macro._last_setval == target and len(gui.macro) > 0:
             gui.macro.pop()
             gui.macro._erase_last()
@@ -851,12 +997,13 @@ def nested_function_gui_callback(gui: MagicTemplate, fgui: FunctionGui):
             # Auto-call will cause many redundant macros. To avoid this, only the last input
             # will be recorded in magic-class.
             last_expr = gui.macro[-1]
-            if (last_expr.head == Head.call and last_expr.args[0].head == Head.getattr and
-                last_expr.args[0].args[1] == expr.args[0].args[1] and
+            if (last_expr.head == Head.call and
+                last_expr.args[0].head == Head.getattr and
+                last_expr.at(0, 1) == expr.at(0, 1) and
                 len(gui.macro) > 0):
                 gui.macro.pop()
                 gui.macro._erase_last()
 
         gui.macro.append(expr)
         gui.macro._last_setval = None
-    return _after_run
\ No newline at end of file
+    return _after_run
