@@ -1,7 +1,10 @@
 from __future__ import annotations
 from inspect import signature
 from typing import Any, Callable
-from qtpy.QtWidgets import QMenuBar, QWidget
+import os
+import warnings
+from qtpy.QtWidgets import QMenuBar, QWidget, QMainWindow, QBoxLayout
+from qtpy.QtGui import QIcon
 from qtpy.QtCore import Qt
 from magicgui.widgets import Container, MainWindow,Label, FunctionGui, Image, Table
 from magicgui.widgets._bases import Widget, ButtonWidget, ValueWidget, ContainerWidget
@@ -9,9 +12,10 @@ from magicgui.widgets._concrete import _LabeledWidget
 from macrokit import Symbol
 
 from .mgui_ext import PushButtonPlus
+from .toolbar import ToolBarGui
 from .menu_gui import MenuGui, ContextMenuGui
 from ._base import BaseGui, PopUpMode, ErrorMode, value_widget_callback, nested_function_gui_callback
-from .utils import define_callback, MagicClassConstructionError
+from .utils import define_callback, MagicClassConstructionError, define_context_menu
 from ..widgets import (
     ButtonContainer,
     GroupBoxContainer,
@@ -20,6 +24,7 @@ from ..widgets import (
     ScrollableContainer,
     DraggableContainer,
     CollapsibleContainer,
+    HCollapsibleContainer,
     SplitterContainer,
     StackedContainer,
     TabbedContainer,
@@ -30,6 +35,7 @@ from ..widgets import (
 from ..utils import iter_members, extract_tooltip
 from ..fields import MagicField
 from ..signature import get_additional_option
+from .._app import run_app
 
 class ClassGuiBase(BaseGui):
     # This class is always inherited by @magicclass decorator.
@@ -42,15 +48,10 @@ class ClassGuiBase(BaseGui):
         cls = self.__class__
         if fld.not_ready():
             try:
-                fld.default_factory = cls.__annotations__[name]
-                if isinstance(fld.default_factory, str):
-                    # Sometimes annotation is not type but str. 
-                    from pydoc import locate
-                    fld.default_factory = locate(fld.default_factory)
-                    
+                fld.decode_string_annotation(cls.__annotations__[name])    
             except (AttributeError, KeyError):
                 pass
-        
+            
         fld.name = fld.name or name.replace("_", " ")
         widget = fld.get_widget(self)
             
@@ -94,12 +95,10 @@ class ClassGuiBase(BaseGui):
                     # Nested magic-class
                     widget = attr()
                     setattr(self, name, widget)
-                    self.__magicclass_children__.append(widget)
                 
                 elif isinstance(attr, BaseGui):
                     widget = attr
                     setattr(self, name, widget)
-                    self.__magicclass_children__.append(widget)
                 
                 elif isinstance(attr, MagicField):
                     # If MagicField is given by field() function.
@@ -115,14 +114,28 @@ class ClassGuiBase(BaseGui):
                     widget = getattr(self, name, None)
                 
                 if isinstance(widget, BaseGui):
+                    widget.__magicclass_parent__ = self
+                    self.__magicclass_children__.append(widget)
                     widget._my_symbol = Symbol(name)
                             
                 if isinstance(widget, MenuGui):
                     # Add menubar to container
                     widget.__magicclass_parent__ = self
                     if self._menubar is None:
+                        # if widget has no menubar, a new one should be created.
                         self._menubar = QMenuBar(parent=self.native)
-                        self.native.layout().setMenuBar(self._menubar)
+                        if issubclass(self.__class__, MainWindow):
+                            self.native: QMainWindow
+                            self.native.setMenuBar(self._menubar)
+                        else:
+                            _layout: QBoxLayout = self.native.layout()
+                            if _layout.menuBar() is None:
+                                _layout.setMenuBar(self._menubar)
+                            else:
+                                raise RuntimeError(
+                                    "Cannot add both toolbar and menubar to a non-main window."
+                                    "Use maigcclass(widget_type='mainwindow') instead."
+                                    )
                     
                     widget.native.setParent(self._menubar, widget.native.windowFlags())
                     self._menubar.addMenu(widget.native)
@@ -133,10 +146,27 @@ class ClassGuiBase(BaseGui):
                     widget.__magicclass_parent__ = self
                     self.native.setContextMenuPolicy(Qt.CustomContextMenu)
                     self.native.customContextMenuRequested.connect(
-                        _define_context_menu(widget, self.native)
+                        define_context_menu(widget, self.native)
                         )
                     _hist.append((name, type(attr), "ContextMenuGui"))
                 
+                elif isinstance(widget, ToolBarGui):
+                    widget.__magicclass_parent__ = self
+                    if issubclass(self.__class__, MainWindow):
+                        self.native: QMainWindow
+                        self.native.addToolBar(widget.native)
+                    else:
+                        # self is not a main window object
+                        _layout: QBoxLayout = self.native.layout()
+                        if _layout.menuBar() is None:
+                            _layout.setMenuBar(widget.native)
+                        else:
+                            raise RuntimeError(
+                                "Cannot add two or more toolbars and/or menubars to a non-main"
+                                "window. Use maigcclass(widget_type='mainwindow') instead."
+                                )
+                    _hist.append((name, type(attr), "ToolBarGui"))
+                
                 elif isinstance(widget, (Widget, Callable)):
                     if (not isinstance(widget, Widget)) and isinstance(widget, Callable):
                         # Methods or any callable objects, but FunctionGui is not included.
@@ -175,7 +205,7 @@ class ClassGuiBase(BaseGui):
                     if clsname is not None:
                         self._unwrap_method(clsname, name, widget)
                     else:
-                        self.insert(n_insert, widget)
+                        self._fast_insert(n_insert, widget)
                         n_insert += 1
                     
                     _hist.append((name, str(type(attr)), type(widget).__name__))
@@ -197,11 +227,10 @@ class ClassGuiBase(BaseGui):
         # convert __call__ into a button
         if hasattr(self, "__call__"):
             widget = self._create_widget_from_method(self.__call__)
-            self.insert(n_insert, widget)
+            self._fast_insert(n_insert, widget)
             n_insert += 1
         
         self._unify_label_widths()
-        del _hist
         return None
 
 def make_gui(container: type[ContainerWidget], no_margin: bool = True):
@@ -239,8 +268,8 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
                 container.__setattr__(self, name, value)
             else:
                 object.__setattr__(self, name, value)
-
-        def insert(self: cls, key: int, widget: Widget):
+        
+        def _fast_insert(self: cls, key: int, widget: Widget):
             # _hide_labels should not contain Container because some ValueWidget like widgets
             # are Containers.
             _hide_labels = (_LabeledWidget, ButtonWidget, ClassGuiBase, FreeWidget, Label,
@@ -267,13 +296,23 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
             if key < 0:
                 key += len(self)
             self._widget._mgui_insert_widget(key, _widget)
-            self._unify_label_widths()
+            
 
+        def insert(self: cls, key: int, widget: Widget):
+            self._fast_insert(key, widget)
+            self._unify_label_widths()
         
-        def show(self: cls, run: bool = False) -> None:
+        
+        def show(self: cls, run: bool = True) -> None:
             """
             Show ClassGui. If any of the parent ClassGui is a dock widget in napari, then this
             will also show up as a dock widget (floating if in popup mode).
+            
+            Parameters
+            ----------
+            run : bool, default is True
+                *Unlike magicgui, this parameter should always be True* unless you want to close
+                the window immediately. If true, application gets executed *if needed*.
             """        
             if self.__magicclass_parent__ is not None and self.parent is None:
                 # If child magic class is closed before, we have to set parent again.
@@ -290,8 +329,10 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
                                                          allowed_areas=["left", "right"])
                     dock.setFloating(self._popup_mode == PopUpMode.popup)
             else:
-                container.show(self, run=run)
+                container.show(self, run=False)
                 self.native.activateWindow()
+            if run:
+                run_app()
             return None
         
         def close(self: cls):
@@ -310,6 +351,7 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
         
         cls.__init__ = __init__
         cls.__setattr__ = __setattr__
+        cls._fast_insert = _fast_insert
         cls.insert = insert
         cls.show = show
         cls.close = close
@@ -334,6 +376,9 @@ class DraggableClassGui: pass
 @make_gui(CollapsibleContainer)
 class CollapsibleClassGui: pass
 
+@make_gui(HCollapsibleContainer)
+class HCollapsibleClassGui: pass
+
 @make_gui(ButtonContainer)
 class ButtonClassGui: pass
 
@@ -357,8 +402,3 @@ class GroupBoxClassGui: pass
 
 @make_gui(MainWindow)
 class MainWindowClassGui: pass
-
-def _define_context_menu(contextmenu, parent):
-    def rightClickContextMenu(point):
-        contextmenu.native.exec_(parent.mapToGlobal(point))
-    return rightClickContextMenu
