@@ -13,6 +13,7 @@ from .containers import (
     ScrollableContainer,
     DraggableContainer,
     CollapsibleContainer,
+    HCollapsibleContainer,
     SplitterContainer,
     StackedContainer,
     TabbedContainer,
@@ -86,6 +87,7 @@ __all__ = ["ListWidget",
            "ScrollableContainer",
            "DraggableContainer",
            "CollapsibleContainer",
+           "HCollapsibleContainer",
            "SplitterContainer",
            "StackedContainer",
            "TabbedContainer",
